'''Script for aggregating the testing result'''
import os
import pandas as pd
from scipy.stats import ttest_ind


# select the datasets for which the results will be calculated
datasets = {
    # folder name: display name
    'ppi': 'PPI',
    'flickr': 'Flickr',
    'blogcat': 'BlogCatalog',
    'rcv1v2': 'RCV1',
    'eurlex': 'EUR-Lex',
    'wiki31k': 'Wiki10-31K',
}

# select and order the methods
methods = [
    ## main experiments
    'binary',
    'SCut',
    'SCutFBR',
    'cost-sensitive',
    'cost-sensitive-micro',
    'SCutFBRn',
    'smooth-each',
    'smooth-all',
    'micromacro',
    ## uncomment these for label order experiments
    # 'micromacro-freq',
    # 'micromacro-tail',
    # 'micromacro-rand',
]

search_dirs = ['./exp_outputs']
output_dir = './exp_outputs'
options = '-q -B 1'
RESULT_NAME = 'result'

# whether to produce an extra files with the standard deviations
STD = True

# pairs of methods to be tested
t_test = [
    ('SCutFBR', 'smooth-all'),
    ('SCutFBR', 'micromacro')
]

#### END OF CONFIGURATION ####


def find_results(paths, data, options):
    results = []
    for path in paths:
        for (dirpath, _, files) in os.walk(path):
            if (f'{RESULT_NAME}.csv' in files and f'/{data}_' in dirpath and 
                f'/({options})' in dirpath):
                results.append(os.path.join(dirpath, f'{RESULT_NAME}.csv'))
    return results

csvs = {}

for data, data_name in datasets.items():
    results = find_results(search_dirs, data, options)
    if not results:
        print(f'No results found for data {data}')
        continue

    dfs = []
    for result in results:
        df = pd.read_csv(result, index_col='method')
        exists = [m in df.index for m in methods]
        if False in exists:
            print(f'WARNING: {[m for e,m in zip(exists, methods) if not e]} are not in {result}.')
        elif df.isna().any(axis=None):    
            print(f'WARNING: results in {result} contains NA')
        else:
            print(f'Read {result}')
        dfs.append(df)

    concated = pd.concat(dfs)*100

    aggregated = concated.groupby(concated.index, sort=False).mean()
    aggregated = (aggregated).round(2).astype(str)
    csv_path = os.path.join(output_dir, f'{data} ({options}) {RESULT_NAME}.csv')
    aggregated.to_csv(csv_path, sep='&', float_format='%.2f') # for latex
    csvs[data_name] = aggregated
    print(f'Outputted {csv_path}.')

    if STD:
        std = concated.groupby(concated.index, sort=False).std().round(2).astype(str)
        with_std = aggregated + "$\pm$" + std
        # with_std = pd.concat([with_std, tests])
        csv_path = os.path.join(output_dir, f'{data} ({options}) {RESULT_NAME} std.csv')
        with_std.to_csv(csv_path, sep='&', float_format='%.2f') 
        csvs[data_name] = with_std
        print(f'Outputted {csv_path}.')

    if t_test:
        tests = pd.DataFrame(columns=['microF', 'macroF'])
        for a,b in t_test:
            a_df = concated.loc[a]
            b_df = concated.loc[b]
            # print(a_df, b_df)
            tests.loc[f'{a} < {b}'] = ttest_ind(a_df, b_df, axis=0, alternative='less', 
                                                equal_var=False, nan_policy='raise').pvalue
        
        tests=(tests * 100).round(2)
        csv_path = os.path.join(output_dir, f'{data} ({options}) {RESULT_NAME} test.csv')
        tests.to_csv(csv_path, sep='&', float_format='%.2f') 
        print(f'Outputted {csv_path}.')
