function [w, bias] = cost_sensitive(train_data, options)
    % cost_sensitive (select parameter -w1 to optimize F-measure for each label)
    % input:
    %   train_data: a struct with `y`,`x`,and `map` return by libsvmread_ml
    %   options: options to pass to LIBLINEAR
    % output:
    %   w: weight of the linear SVMs. The shape is (#dim, #label)
    %   bias: the -B value passed to LIBLINEAR. -1 if not passed

    % check input
    if (nargin ~= 2)
        if nargin < 2
            options = '-q -B 1';
        end
        if nargin < 1 || nargin > 2
            fprintf('Incorrect number of input parameters\n');
            return;
        end
    end
    
    x = train_data.x;
    y = train_data.y;
    map = train_data.map;

    % bias option
    bias = get_bias(options);
    if bias > 0
        w_size = size(x, 1) + 1;
    else
        w_size = size(x, 1);
    end

    t0all = clock;
    
    % number of label = number of model to be trained
    L = size(y, 2); 
    
    w = zeros(w_size, L);
    parfor i = 1:L
        fprintf(1, 'INFO: training %dth label (%d)...', i, map(i));
        t0 = clock;
        [w(:, i)] = cost_sensitive_one(y(:, i)*2 - 1, x, options);
        fprintf(1, '\tDone in %.6f seconds\n', etime(clock, t0));
    end
    fprintf(1, 'INFO: Total Time: %.6f seconds\n', etime(clock, t0all));
end

function [w] = cost_sensitive_one(proby, probx, options)
    % perform CV to select the best threshold in terms of macro
    % inputs:
    %   proby: vector of size (#sample,1), elements are -1,+1
    %   probx: feature matrix of size (#feature, #sample)
    %   options: options to be pass to LIBLINEAR

    nr_fold = 3;
    param_space = [1, 1.33, 1.8, 2.5, 3.67, 6, 13];
    best_param = 1;
    best_F = -1;

    bias = get_bias(options);

    warning('off', 'stats:cvpartition:KFoldMissingGrp');
    cv = cvpartition(proby, 'KFold', nr_fold, 'Stratify', true);

    % cv
    for param = param_space
        pred = zeros(numel(proby),1);
        for fold = 1:nr_fold
            train_id = cv.training(fold);
            valid_id = cv.test(fold);

            train_y = proby(train_id);
            train_x = probx(:, train_id);
            valid_x = probx(:, valid_id);

            % scut
            scut_w = do_train(train_y, train_x, [options ' -w1 ' num2str(param)]);
            if bias > 0
                valid_x = [valid_x; ones(1, size(valid_x, 2)) * bias];
            end
            pred(valid_id) = valid_x' * scut_w; % shape = (#sample in valid)
        end
        F = fmeasure(proby, 2*(pred > 0) - 1);
        if F > best_F
            best_param = param;
            best_F = F;
        end 
    end

    w = do_train(proby, probx, [options ' -w1 ' num2str(best_param)]);
end

