
function [w, T_list] = train_for_ab(proby, probx, a_list, b_list, options, train_final)
    % SCut using smoothed F-measure
    % for the provided pairs of (a,b)
    % return a model and the corresponding thresholds
    %
    % proby: vector of size (#sample,1), elements are -1,+1
    % probx: feature matrix of size (#feature, #sample)
    % a_list: a list of parameter `a` to be tried
    % b_list: a list of parameter `b` to be paired with a_list
    %         b_list should be of the same length as a_list
    % options: options to be passed to LIBLINEAR
    % train_final: whether to train the final model using all data

    assert(isequal(size(a_list), size(b_list)))
    T_list = zeros(numel(a_list), 1); % thresholds

    nr_fold = 3;

    bias = get_bias(options);
    l = size(proby, 1);

    warning('off', 'stats:cvpartition:KFoldMissingGrp');
    cv = cvpartition(proby, 'KFold', nr_fold, 'Stratify', true);

    % cv
    for fold = 1:nr_fold
        train_id = cv.training(fold);
        valid_id = cv.test(fold);

        train_y = proby(train_id);
        train_x = probx(:, train_id);
        valid_y = proby(valid_id);
        valid_x = probx(:, valid_id);

        w = do_train(train_y, train_x, options);
        if (bias > 0)
            valid_x = [valid_x; ones(1, size(valid_x, 2)) * bias];
        end
        wTx = valid_x' *  w; % shape = (#sample in valid)

        [sorted_wTx, sorted_wTx_index] = sort(wTx, 1, 'ascend'); 

        % best_F is initialized as predicting all as 1,
        % that is, the threshhold is set to the lowest
        tp = sum(valid_y == 1); fp = size(valid_y,1)-tp; fn = 0;
        cut = repmat(-1, 1, numel(a_list));
        best_F = (a_list + 2*tp) ./ (b_list + 2*tp + fp + fn);
        for i = 1:size(valid_y, 1)
            if valid_y(sorted_wTx_index(i)) == -1,
                fp = fp - 1;
            else
                tp = tp - 1; fn = fn + 1;
            end
            F = (a_list + 2*tp) ./ (b_list + 2*tp + fp + fn);
            
            select = F >= best_F;
            best_F(select) = F(select);
            cut(select) = i;
        end

        for i = 1:numel(T_list)
            if cut(i) == -1 % i.e., all +1
                val = sorted_wTx(1);
                T_list(i) = T_list(i) + (val - eps(val)); % predict all +1
            elseif cut(i) == size(valid_y, 1)
                val = sorted_wTx(size(valid_y, 1));
                T_list(i) = T_list(i) + (val + eps(val));
            else
                T_list(i) = T_list(i) + (sorted_wTx(cut(i)) + sorted_wTx(cut(i)+1))/2;
            end
        end
    end

    % final model
    T_list = T_list / nr_fold;
    if train_final
        w = do_train(proby, probx, options);
    else
        if bias > 0
            w_size = size(probx, 1) + 1;
        else
            w_size = size(probx, 1);
        end
        w = zeros(w_size, 1);
    end
end