function run_exp(data_name, options, rand_seed, methods)
    % data_name: a string that is a key of the Map datasets
    % options: options to be passed to LIBLINEAR 
    % rand_seed: every algorithm is seeded with rand_seed before running
    % methods: a cell array of methods that will be ran
    addpath('utils', 'algos');

    % default arguments
    if nargin > 4 || nargin < 1
        fprintf(1, 'Incorrect number of arguments\n')
        return;
    end
    if nargin < 4
        methods = {...
            'SCut', 'SCutFBR', 'SCutFBRn',...
            'smooth-all', 'smooth-each',...
            'micromacro', 'cost-sensitive',...
            'cost-sensitive-micro'...
        };
    end
    if nargin < 3
        rand_seed = 1;
    end
    if nargin < 2
        options = '-q -B 1';
    end

    methods = lower(methods);

    run('data_paths.m') % import the `datasets` Map, which records the path of files

    train_test = datasets(data_name);
    train_path = train_test{1};
    test_path = train_test{2};
    output_dir = ['exp_outputs/' data_name '_' num2str(rand_seed) '/(' options ')/'];
    mkdir(output_dir);
    result_path = [output_dir 'result.csv'];
    if isfile(result_path)
        result = readtable(result_path);
        result.Properties.RowNames = result.method;
    else
        result = table('Size', [0,3], 'VariableTypes', {'cellstr', 'double', 'double'});
        result.Properties.VariableNames = {'method','microF','macroF'};
    end

    if strcmp(data_name,'ppi') || strcmp(data_name, 'blogcat') || strcmp(data_name, 'flickr')
        options = [options ' -s 2']; % use primal solver
    end

    t0all = clock;

    [y, x, map] = libsvmread_ml(train_path);
    [ty, tx, tmap] = libsvmread_ml(test_path);

    x = x';
    tx = tx';

    bias = get_bias(options);
    [x, tx] = fill_x(x, tx);
    if bias > 0
        % add bias term for test data
        % train data is handled by liblinear
        tx=[tx; ones(1, size(tx,2)) * bias];
    end

    % remove duplicate labels
    y = sparse(full(y));
    ty = sparse(full(ty));

    train_data = struct('y', y, 'x', x, 'map', map);
    test_data = struct('y', ty, 'x', tx, 'map', tmap);

    %% binary %%
    % This will always be ran
    % the final linear model is not affected by the thresholding strategy
    % we train it only once and re-use it for other methods to speed up the experiment
    rand('seed', rand_seed);
    [w, bias] = binary(train_data, options);
    pre = tx' * w;
    [micro, macro] = f1_score(ty, pre, tmap, map);
    clear pre;
    % save model
    save([output_dir 'binary.mat'], 'w', 'bias', '-v7.3');
    % save metric
    result('binary',:) = {'binary', micro, macro};
    writetable(result, result_path);

    %% SCut with no fbr (fbr = -1) %%
    if any(strcmp('scut', methods))
        rand('seed', rand_seed);
        [~, T, bias] = SCut(train_data, options, -1, false);
        pre = tx' * w - T';
        [micro, macro] = f1_score(ty, pre, tmap, map);
        clear pre;
        % save model and metric
        save([output_dir 'SCut.mat'], 'T', 'bias', '-v7.3');
        result('SCut',:) = {'SCut', micro, macro};
        writetable(result, result_path);
    end

    %% SCutFBR with parameter selection of fbr
    if any(strcmp('scutfbr', methods))
        rand('seed', rand_seed);
        [~, T, bias] = SCutFBR1(train_data, options, false);
        pre = tx' * w - T';
        [micro, macro] = f1_score(ty, pre, tmap, map);
        clear pre;
        % save model and metric
        save([output_dir 'SCutFBR.mat'], 'T', 'bias', '-v7.3');
        result('SCutFBR',:) = {'SCutFBR', micro, macro};
        writetable(result, result_path);
    end

    %% SCutFBR1 but set to max neg
    if any(strcmp('scutfbrn', methods))
        rand('seed', rand_seed);
        [~, T, bias] = SCutFBRn(train_data, options, false);
        pre = tx' * w - T';
        [micro, macro] = f1_score(ty, pre, tmap, map);
        clear pre;
        % save model and metric
        save([output_dir 'SCutFBRn.mat'], 'T', 'bias', '-v7.3');
        result('SCutFBRn',:) = {'SCutFBRn', micro, macro};
        writetable(result, result_path);
    end

    %% smooth-all
    if any(strcmp('smooth-all', methods))
        rand('seed', rand_seed);
        [~, T, bias, best_a, best_b, a_list, b_list, cv_F] = smooth_all(train_data, options, false);
        pre = tx' * w - T';
        [micro, macro] = f1_score(ty, pre, tmap, map);
        clear pre;
        % save model and metric
        save([output_dir 'smooth-all.mat'], 'T', 'bias', ...
            'best_a', 'best_b', 'a_list', 'b_list', 'cv_F', '-v7.3');
        result('smooth-all',:) = {'smooth-all', micro, macro};
        writetable(result, result_path);
    end

    %% smooth-each
    if any(strcmp('smooth-each', methods))
        rand('seed', rand_seed);
        [~, T, bias, best_as, best_bs] = smooth_each(train_data, options, false);
        pre = tx' * w - T';
        [micro, macro] = f1_score(ty, pre, tmap, map);
        clear pre;
        % save model and metric
        save([output_dir 'smooth-each.mat'], 'T', 'bias', 'best_as', 'best_bs', '-v7.3');
        result('smooth-each',:) = {'smooth-each', micro, macro};
        writetable(result, result_path);
    end

    %% micromacro
    if any(strcmp('micromacro', methods))
        rand('seed', rand_seed);
        [~, T, bias] = micromacro(train_data, options, false);
        pre = tx' * w - T';
        [micro, macro] = f1_score(ty, pre, tmap, map);
        clear pre;
        % save model and metric
        save([output_dir 'micromacro.mat'], 'T', 'bias', '-v7.3');
        result('micromacro',:) = {'micromacro', micro, macro};
        writetable(result, result_path);
    end

    %% micromacro-freq
    if any(strcmp('micromacro-freq', methods))
        rand('seed', rand_seed);
        [~, T, bias] = micromacro_freq(train_data, options, false);
        pre = tx' * w - T';
        [micro, macro] = f1_score(ty, pre, tmap, map);
        clear pre;
        % save model and metric
        save([output_dir 'micromacro-freq.mat'], 'T', 'bias', '-v7.3');
        result('micromacro-freq',:) = {'micromacro-freq', micro, macro};
        writetable(result, result_path);
    end

    %% micromacro-tail
    if any(strcmp('micromacro-tail', methods))
        rand('seed', rand_seed);
        [~, T, bias] = micromacro_tail(train_data, options, false);
        pre = tx' * w - T';
        [micro, macro] = f1_score(ty, pre, tmap, map);
        clear pre;
        % save model and metric
        save([output_dir 'micromacro-tail.mat'], 'T', 'bias', '-v7.3');
        result('micromacro-tail',:) = {'micromacro-tail', micro, macro};
        writetable(result, result_path);
    end

    %% micromacro-rand
    if any(strcmp('micromacro-rand', methods))
        rand('seed', rand_seed);
        [~, T, bias] = micromacro_rand(train_data, options, false);
        pre = tx' * w - T';
        [micro, macro] = f1_score(ty, pre, tmap, map);
        clear pre;
        % save model and metric
        save([output_dir 'micromacro-rand.mat'], 'T', 'bias', '-v7.3');
        result('micromacro-rand',:) = {'micromacro-rand', micro, macro};
        writetable(result, result_path);
    end

    %% cost-sensitive
    if any(strcmp('cost-sensitive', methods))
        rand('seed', rand_seed);
        [w, bias] = cost_sensitive(train_data, options);
        pre = tx' * w;
        [micro, macro] = f1_score(ty, pre, tmap, map);
        clear pre;
        % save model and metric
        save([output_dir 'cost-sensitive.mat'], 'w', 'bias', '-v7.3');
        result('cost-sensitive',:) = {'cost-sensitive', micro, macro};
        writetable(result, result_path);
    end

    %% cost-sensitive-micro
    if any(strcmp('cost-sensitive-micro', methods))
        rand('seed', rand_seed);
        [w, bias] = cost_sensitive_micro(train_data, options);
        pre = tx' * w;
        [micro, macro] = f1_score(ty, pre, tmap, map);
        clear pre;
        % save model and metric
        save([output_dir 'cost-sensitive-micro.mat'], 'w', 'bias', '-v7.3');
        result('cost-sensitive-micro',:) = {'cost-sensitive-micro', micro, macro};
        writetable(result, result_path);
    end
    
    fprintf(1, 'INFO: Done in %.6f seconds\n', etime(clock, t0all));
end
